#include "interface.h"

gboolean expose_ev(GtkWidget *widget, GdkEventExpose *event, gpointer user_data);


GtkWidget *create_interface(){
	
	GtkWidget *window;
	GtkWidget *boxMain;
	GtkWidget *menuBar;
	GtkWidget *hBox;
	GtkWidget *toolbox;
	GtkWidget *separator;
	GtkWidget *darea;
	GdkPixbuf *icnpixbuf;

	icnpixbuf = gdk_pixbuf_new_from_file("imgs/mmfm.png",NULL);

	if (!icnpixbuf) {
		intf_alert("Could not load the image imgs/mmfm.png!");
		return NULL;
	}
	accelGroup = gtk_accel_group_new();


	/*
	 *  vytvorime vsechny prvky GUI a nastavime jim pozadovane parametry
	 */

	window = intf_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	  {gtk_window_set_title((GtkWindow*)window, APP_NAME);}
	  {gtk_window_set_icon((GtkWindow*)window, icnpixbuf);}
	  {gtk_window_set_default_icon(icnpixbuf);}
	  {gtk_window_set_position((GtkWindow*)window, GTK_WIN_POS_CENTER);}
	  {gtk_widget_set_size_request(window, 10, 10);}
	  {gtk_window_maximize((GtkWindow*)window);}
	  {gtk_window_set_default_size(GTK_WINDOW(window), WINDOW_WIDTH, WINDOW_HEIGHT);}
	  {gtk_window_add_accel_group(GTK_WINDOW(window), accelGroup);}
	  	boxMain = gtk_vbox_new(FALSE, 0);
	  		menuBar = gui_menu_menubar();
			hBox = gtk_hbox_new(FALSE, 0);
				toolbox = gui_toolbox_toolbox();
				separator = gtk_vseparator_new();
				darea = intf_darea = gtk_drawing_area_new();

	/*
	 *  pospojujeme vsechny prvky do sebe
	 */

	gtk_container_add(GTK_CONTAINER(window), boxMain);
		gtk_box_pack_start(GTK_BOX(boxMain), menuBar, FALSE, FALSE, 0);
		gtk_box_pack_start(GTK_BOX(boxMain), hBox, TRUE, TRUE, 0);
			gtk_box_pack_start(GTK_BOX(hBox), toolbox, FALSE, FALSE, 10);
			gtk_box_pack_start(GTK_BOX(hBox), separator, FALSE, TRUE, 0);
			gtk_box_pack_start(GTK_BOX(hBox), darea, TRUE, TRUE, 0);

	/*
	 *  pripojime obsluhy udalosti
	 */


	g_signal_connect((GObject*)darea, "expose-event", (GCallback)expose_ev, NULL);
	g_signal_connect((GObject*)window, "destroy", (GCallback)destroy_ev, NULL);
	
	return window;
}

gboolean expose_ev(GtkWidget *widget, GdkEventExpose *event, gpointer user_data){
	if (rgbBuffer == NULL)
		return FALSE;
	paint_start();
	gdk_draw_rgb_image(	intf_darea->window,
					intf_darea->style->fg_gc[GTK_STATE_NORMAL],
					0, 0,
					parameter.width, parameter.height,
					GDK_RGB_DITHER_MAX,
					rgbBuffer,
					parameter.width * 3);
	return TRUE;
}

void refresh() {
	int i;
	
	/*vynulovani bufferu...cerna barva*/
	for(i=0; i<parameter.width * parameter.height * 3; i++)
		rgbBuffer[i] = 0;
	
	frac_draw(parameter.fractal);
}

/*
 *    Reset Modes:
 *        0 - reset all
 *        1 - reset move
 *        2 - reset zoom
 */

void reset(int mode) {
	struct Parameter tmp;
	frac_reset(parameter.fractal, &tmp);
	
	if (mode == 1) {
		parameter.x = tmp.x;
		parameter.y = tmp.y;
	} else if (mode == 2) {
		parameter.zoom = tmp.zoom;
	} else {
		parameter.alpha = tmp.alpha;
		parameter.beta = tmp.beta;
		parameter.iters = tmp.iters;
		parameter.zoom = tmp.zoom;
		parameter.x = tmp.x;
		parameter.y = tmp.y;
		parameter.color = tmp.color;
	}
	
	refresh();
	refresh_values();
	gtk_widget_queue_draw(intf_darea);

}

void refresh_values() {
	gtk_spin_button_set_value((GtkSpinButton*)spinButtonA, parameter.alpha);
	gtk_spin_button_set_value((GtkSpinButton*)spinButtonB, parameter.beta);
	gtk_spin_button_set_value((GtkSpinButton*)spinButtonIter, parameter.iters);
	gtk_spin_button_set_value((GtkSpinButton*)spinButtonZoom, parameter.zoom);
	gtk_toggle_button_set_active((GtkToggleButton*)checkColor, parameter.color);
	gtk_check_menu_item_set_active((GtkCheckMenuItem*)mniFractalColor, parameter.color);
}

void put_pixel(double x, double y, int iter){ 

	guchar colorR;
	guchar colorG;
	guchar colorB;
	
	gint color;
	
	x = floor(x);
	y = floor(y);
	
	if (x<0 || y<0 || x > parameter.width || y > parameter.height)
		return;
	if (((y*parameter.width + x)*3 + 2) > limit)
		return;
	if (((y*parameter.width + x)*3) < 0)
		return;
	
	if (parameter.color == FALSE) {
		colorR = ((double)iter/parameter.iters)*255;
		colorG = ((double)iter/parameter.iters)*255;
		colorB = ((double)iter/parameter.iters)*255;
	} else {
		color = ((double)iter/parameter.iters)*(256*256*256-1);
		colorR = color % 256;
		color /= 256;
		colorG = color % 256;
		color /= 256;
		colorB = color;
	}

	rgbBuffer[(gint)((y*parameter.width + x)*3)]     = colorR;
	rgbBuffer[(gint)((y*parameter.width + x)*3 + 1)] = colorG;
	rgbBuffer[(gint)((y*parameter.width + x)*3 + 2)] = colorB;
}


/*
 *   ulozi aktualni rozmery platna
 */
void compute_size() {
	gint oldw = parameter.width;
	gint oldh = parameter.height;
	
	gdk_window_get_geometry(	intf_darea->window,
								NULL,NULL,
								&parameter.width,&parameter.height,
								NULL);

	if(oldw!=parameter.width || oldh!=parameter.height){
		rgbBuffer = (guchar*)realloc(rgbBuffer,parameter.width*parameter.height*3*sizeof(char));
		limit = parameter.width*parameter.height*3;
		refresh();
	}
}

/*
 *   ulozi aktualni rozmery a vyplni platno cernou barvou
 */
void paint_start() {
	compute_size();
	
	gdk_draw_rectangle(	intf_darea->window,
						intf_darea->style->fg_gc[GTK_STATE_NORMAL],
						TRUE,
						0,0,
						parameter.width, parameter.height);
	
}

/*
 *   vyhodi msgbox upozornujici, ze neco nejde
 */
void intf_alert(char * zprava) {
	GtkWidget *dialog;
	
	dialog = gtk_message_dialog_new(	(GtkWindow *)intf_window,
										GTK_DIALOG_MODAL,
										GTK_MESSAGE_WARNING,
										GTK_BUTTONS_OK,
										(gchar*)zprava);
	gtk_window_set_title((GtkWindow*)dialog, "Warning");
	gtk_dialog_run((GtkDialog*)dialog);
	gtk_widget_destroy(dialog);
}

/*
 *   vyhodi msgbox upozornujici, ze neco nejde a skonci program
 */
void intf_alert_fatal(char * zprava) {
	intf_alert(zprava);
	main_quit();
}
